package com.chrismweb.box2dforflashgames
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.display.Sprite
	import Box2D.Dynamics.*;
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	
	/**
	 * ...
	 * #Chp2.1 - Ball drop with bounce
	 * @book by Emanuele Feronato
	 * http://www.emanueleferonato.com/
	 * Get it now @ http://www.packtpub.com/box2d-for-flash-games/book
	 */
	public class Main extends Sprite 
	{
		private var world:b2World;
		private var worldScale:Number = 30;
		
		public function Main():void 
		{
			if (stage) 
				Init();
			else 
				addEventListener(Event.ADDED_TO_STAGE, Init);
		}
		
		private function Init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, Init);
			// entry point

			var gravity:b2Vec2 = new b2Vec2(0, 9.81);
			var sleep:Boolean = true;
			world = new b2World(gravity, sleep); //need to make sure you removed 'var', otherwise it will only initialize it as a local variable
			
			Setup();
			
			//for enabling debug draw
			var debugSprite:Sprite = new Sprite();
			addChild(debugSprite);
			
			var debugDraw:b2DebugDraw = new b2DebugDraw();
			debugDraw.SetSprite(debugSprite);
			debugDraw.SetDrawScale(worldScale);
			debugDraw.SetFlags(b2DebugDraw.e_shapeBit);
			debugDraw.SetFillAlpha(0.5);
			world.SetDebugDraw(debugDraw);
			
			
			
			
			//stuff needed to reset example
			var reset_txt:TextField = new TextField();			
			var format:TextFormat = new TextFormat();
			format.color = 0xFFFFFFFF;
			format.align = TextFormatAlign.CENTER;
			format.size = 20;
			format.font = "Arial";
			
			reset_txt.width = 100;
			reset_txt.height = 30;
			reset_txt.selectable = false;
			reset_txt.text = "Reset";
			reset_txt.textColor = 0xFFFFFFFF;
			reset_txt.backgroundColor = 0xFF0D3451;
			reset_txt.opaqueBackground = reset_txt.backgroundColor;
			reset_txt.border = true;
			reset_txt.borderColor = 0xFFFFFFFF;
			reset_txt.defaultTextFormat = format;
			reset_txt.setTextFormat(format);
			addChild(reset_txt);
			reset_txt.addEventListener(MouseEvent.CLICK, function(e:MouseEvent):void { ResetWorld() } );
			
			
			
			addEventListener(Event.ENTER_FRAME,UpdateWorld);
		}
		
		private function ResetWorld():void
		{
			trace("Resetting world");
			for (var body:b2Body=world.GetBodyList();body!=null; body=body.GetNext())
				world.DestroyBody(body);
				
			Setup();
		}
		private function Setup():void
		{
			//for creating a circle
			var bodyDef:b2BodyDef=new b2BodyDef();
			bodyDef.position.Set(PixToMeter(320), PixToMeter(30));
			bodyDef.type = b2Body.b2_dynamicBody;
			
			var circleShape:b2CircleShape;
			circleShape=new b2CircleShape(25/worldScale);
			var fixtureDef:b2FixtureDef = new b2FixtureDef();
			fixtureDef.shape = circleShape;
			fixtureDef.density=1;
			fixtureDef.restitution=0.6;
			fixtureDef.friction=0.1;
			var theBall:b2Body=world.CreateBody(bodyDef);
			theBall.CreateFixture(fixtureDef);
			
			
			//for creating the "floor" bottom rectangle
			bodyDef.type=b2Body.b2_staticBody;
			bodyDef.position.Set(PixToMeter(320), PixToMeter(470));//center location of the box- 320 to the right (640/2), and 10px from the bottom(480)
			var polygonShape:b2PolygonShape=new b2PolygonShape();
			polygonShape.SetAsBox(PixToMeter(320), PixToMeter(10));//1/2 width and height, so 640x20 px box
			fixtureDef.shape=polygonShape;
			var theFloor:b2Body=world.CreateBody(bodyDef);
			theFloor.CreateFixture(fixtureDef);
		}
		
		private function PixToMeter(pixels:Number):Number
		{
			return pixels / worldScale;
		}
		private function UpdateWorld(e:Event):void 
		{
			var timeStep:Number = 1 / 30;
			var velIterations:int=10;
			var posIterations:int=10;
			world.Step(timeStep, velIterations, posIterations);
			world.ClearForces();
			world.DrawDebugData();
		}
	}
	
}